program TimerDemo ;
{ Illustrates the use of the Timers.dll library }
{ Digital Optics Ltd  2000 }

{ Timers.dll provides Windows timer services to VPascal modules by
  calling DDE shared procedures on timer events. This enables you
  to trigger an action at a regular interval without user intervention.
  
  You can create multiple timers each of which calls a specific
  shared procedure. Refer to the V++ online help for more information
  on DDE and shared procedures.
  
  The procedure names must refer to VPascal shared procedures but the
  procedures themselves do not have to be in the module that created
  the timer.
  
  - SetDdeTimer
    Creates a timer that executes a shared procedure (Name) at a
    fixed time interval (Elapse). Returns a timer id that should
    be stored by the module. The id is not the same as a Windows
    timer handle.
    
  - KillDdeTimer
    Halts and deletes a timer created by SetDdeTimer. The Id
    parameter must be the timer id returned by SetDdeTimer.
    
  - KillAllTimers
    Halts and deletes all timers.
	
  Windows timers are a limited system resource so you should always 
  kill them when they are no longer required.
  
  Windows limits the time resolution of standard timers to no less
  than 55 ms. In other words, your timer events cannot occur more
  frequently than about 18 times a second.
  
  In practice, we recommend that you use repeat times of 1000 ms or
  more to avoid impacting system performance.  
  
  You should copy the Timers.dll file to your V++ executable files
  folder (usually C:\Program Files\Digital Optics\V++\Executable).
  Otherwise, you will have to modify the TImersLib constant below.
  
}
  
const
  { Path to DLL file }
  TimersLib = 'Timers.dll' ;
  

{ Declare external routines }
function SetDdeTimer( Elapse:integer; Name:pointer ) : integer ;
external TimersLib ;

procedure KillDdeTimer( Id:integer ) ; 
external TimersLib ;

procedure KillAllTimers ; 
external TimersLib ;


var
  TimerId ;
  Tau ;
  n ;
  
  
procedure Update ; dde 'Update' ;
{ This is the shared procedure that is triggered by the timer }

begin
  n := n + 1 ;
  WriteStatus( 'Number of updates: ',n ) ;
end; 
  
  
procedure Start ;
button btn_Stopwatch,'Start timer' ;
{ Creates a timer that calls the shared routine 'Update' }

begin
  if TimerId = -1 then
    TimerId := SetDdeTimer( Tau,'Update' ) ;
end;


procedure Stop ;
button btn_Hand,'Stop timer' ;
{ Halts and deletes the current timer }

begin
  if TimerId >= 0 then begin
    KillDdeTimer( TimerId ) ;
	TimerId := -1 ;
  end;
end;


procedure SetDelay ;
button btn_Tau,'Set time interval' ;
{ Changes the timer repeat period }
{ Restarts the timer if necessary }

begin
  if GetNumber( 'Enter a time interval (milliseconds):',Tau ) = id_Ok then
    if TimerId >= 0 then begin
	  Stop ;
	  Start ;
	end;
end;


procedure CleanUp ; shutdown ;
{ Kills all timers on V++ shutdown }

begin
  KillAllTimers ;
end;


begin
  TimerId := -1 ;
  Tau := 3000 ;
  n := 0 ;
end.
